#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a painting on satin

# (c) Dinasset 2021
#
#   History:
#
#   v1.0: 2021-12: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
FixWH=2000   

############################################################################
# Service routines
#---------------------------------------------------------------------------
  
#---------------------------------------- 
# for messages use: erro_msg or info_msg
#---------------------------------------- 
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    pdb.gimp_message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    pdb.gimp_message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

#----------------------------------------------- 
# from Original drawable create the Output image
#----------------------------------------------- 
def createWrkimage(inImage, inLayer):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    OrigWH = inLayer.width+inLayer.height    
    
   # define/calculate the scale factor for the (new) output image  
    scale = (FixWH*1.0)/(OrigWH*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size  
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
     
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
   
    newBG.opacity = 100.0
    
    # adjust mean values
    adjustmeanv (newImage, newBG)
    # equalize 
    #pdb.gimp_drawable_equalize (newBG, False)
    
    return(newImage)
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# adjust values and saturation
#---------------------------------------------------------------------------
def adjustmeanSV (inImage, inLayer):
    # decompose HSV
    (HSVI,HL,SL,VL) = pdb.plug_in_decompose (inImage, inLayer, "HSV", True)
    #pdb.gimp_display_new (HSVI)
    HL=HSVI.layers[0]; SL=HSVI.layers[1]; VL=HSVI.layers[2];
    adjustmeanv (HSVI, SL)
    adjustmeanv (HSVI, VL)
    pdb.plug_in_recompose (HSVI, HL)
    return(inLayer)

#---------------------------------------------------------------------------
# adjust values
#---------------------------------------------------------------------------
def adjustmeanv(inImage, inLayer):
    # take care of the existing mean in determining the theoretical mean
    meanadj=50
    # check mean value of the layer
    (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.0, 1.0)
    #info_msg ("values mean="+str(meanl))
    qobj = 127.5*meanadj/100
    qsrc = meanl*(100.0-meanadj)/100
    thmean = qobj+qsrc   
    #info_msg ("q_obj,q_src,target="+str(qobj)+","+str(qsrc)+","+str(thmean))
    # range for the loops
    minmean = thmean*1.01  
    maxmean = thmean*0.99  
    
    modified=False
    if meanl < minmean and thmean!=meanl:
        ratio = thmean/meanl*1.0 
        #info_msg("mean value found = "+str(meanl)+" ratio = "+str(ratio))
        loop=0
        while meanl < minmean and loop<10:  
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE, 8,
                                             (0.0,0.0,0.3,0.3*ratio, 0.7,0.7*ratio, 1.0,1.0))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
            loop+=1
        modified=True
        #info_msg("increased mean value = "+str(meanl))
    elif meanl > maxmean and thmean!=meanl:
        ratio = thmean/meanl*1.0 
        #info_msg("mean value found = "+str(meanl))
        loop=0
        while meanl > maxmean and loop<10:  
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE, 8,
                                             (0.0,0.0,0.3,0.3*ratio, 0.7,0.7*ratio, 1.0,1.0))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
            loop+=1
        mofified=True
        #info_msg("reduced mean value = "+str(meanl))

    return(inLayer)

############################################################################
# Main
#---------------------------------------------------------------------------

def FakePissarro2 (inImage, inLayer,
                  UserOutWH, NrOfCol, details,
                  doFlatten):
    
    
    # here the logic of the filter
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    OrigWH = inLayer.width+inLayer.height    
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    # create new image to work on
    newImage = createWrkimage (inImage, inLayer)
    width = newImage.width
    height = newImage.height
    Preprocessed = newImage.layers[0]
    Preprocessed.name = "Preprocessed"
    Original = newImage.layers[1]
    
    #pdb.gimp_display_new (newImage)
    #return
      
    # Apply samj_Test_Cubisme_B 64,6,0,0,0,64,6,0,0,0,1,3,1,1,1,25,8,1,0 (1)  
    Cubism1 = Preprocessed.copy()
    newImage.add_layer (Cubism1, 0)
    # for this process remove extremes
    pdb.gimp_drawable_curves_spline (Cubism1, HISTOGRAM_VALUE, 4,
                                     (0.0,0.1,1.0,0.9))
    rotation = (30/360.0)*3.14159 	#converting to radians
    rotated=pdb.gimp_item_transform_rotate (Cubism1, rotation, True,
                                            width/2, height/2)
    Cubism1parms = '64,6,0,0,0,64,6,0,0,0,1,3,1,1,1,25,8,1,0'
    pdb.plug_in_gmic_qt(newImage, Cubism1, 1,0,
    '-v - -samj_Test_Cubisme_B '+Cubism1parms+' ')
    Cubism1 = newImage.layers[0]
    Cubism1.mode = LAYER_MODE_NORMAL
    Cubism1.opacity = 100.0  
    Cubism1.name = 'samj_Test_Cubisme_B '+Cubism1parms
    rotation = (-30/360.0)*3.14159 	#converting to radians
    rotated=pdb.gimp_item_transform_rotate (Cubism1, rotation, True,
                                            width/2, height/2)
    
    #pdb.gimp_display_new (newImage)
    #return
      
    # Apply samj_Test_Cubisme_B 64,6,0,0,0,64,6,0,0,0,0,3,1,1,1,25,8,1,00 (2)
    Cubism2 = Preprocessed.copy()
    newImage.add_layer (Cubism2, 0)
    # for this process remove extremes
    pdb.gimp_drawable_curves_spline (Cubism2, HISTOGRAM_VALUE, 4,
                                     (0.0,0.1,1.0,0.9))
    rotation = (70/360.0)*3.14159 	#converting to radians
    rotated=pdb.gimp_item_transform_rotate (Cubism2, rotation, True,
                                            width/2, height/2)
    Cubism2parms = '64,6,0,0,0,64,6,0,0,0,0,3,1,1,1,25,8,1,0'
    pdb.plug_in_gmic_qt(newImage, Cubism2, 1,0,
    '-v - -samj_Test_Cubisme_B '+Cubism2parms+' ')
    Cubism2 = newImage.layers[0]
    Cubism2.mode = LAYER_MODE_LUMA_DARKEN_ONLY
    Cubism2.opacity = 100.0  
    Cubism2.name = 'samj_Test_Cubisme_B '+Cubism2parms
    rotation = (-70/360.0)*3.14159 	#converting to radians
    rotated=pdb.gimp_item_transform_rotate (Cubism2, rotation, True,
                                            width/2, height/2)
    
    #pdb.gimp_display_new (newImage)
    #return
     
    # Visible
    PreOutcome = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                  "Preliminary")
    newImage.add_layer (PreOutcome, 0)
    PreOutcome.mode = LAYER_MODE_NORMAL
    PreOutcome.opacity = 100.0 
    adjustmeanv(newImage, PreOutcome)
    
    # Source on Top for texture
    pdb.gimp_item_set_visible (Original, True)  # temp for copy
    SourceOnTop = Original.copy()
    newImage.add_layer (SourceOnTop, 0)
    pdb.gimp_item_set_visible (Original, False) # reset 
    pdb.gimp_drawable_posterize (SourceOnTop, str(int(NrOfCol)))
    
    # addition for v.2
    # Apply VectorPaint
    pdb.plug_in_gmic_qt(newImage, SourceOnTop, 1,0 ,
    #[G'MIC] VectorPaint: fx_vector_painting (user),0,50,50
	    "-v - -fx_vector_painting "+str(details)+",0,50,50 ")  #MrQ 
    SourceOnTop = newImage.layers[0]
    SourceOnTop.name = "Source on top (posterized+VectorPaint)"
    SourceOnTop.opacity = 100.0
    SourceOnTop.mode = LAYER_MODE_HARDLIGHT  #GRAIN_MERGE
    
    PreOutcome2 = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                  "Preliminary 2 - level adj on visible")
    newImage.add_layer (PreOutcome2, 0)
    PreOutcome2.mode = LAYER_MODE_NORMAL
    PreOutcome2.opacity = 100.0
    #adjustmeanv(newImage, PreOutcome2)
    
    # reduce saturation (Lula suggestion)
    pdb.gimp_drawable_levels (PreOutcome2, HISTOGRAM_VALUE,
                              0.0, 1.0, False, 1.30, 0.0, 1.0, False)
    
    EFFECT = " Fake Pissarro (2)"
    PARMS = " colours="+str(int(NrOfCol))+", level of details="+str(details)
    
    # final step
    if doFlatten==True:
        outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_BOTTOM_LAYER)
        outcome.name = "Outcome" 
    else:    
        outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
        newImage.add_layer (outcome, 0)
        pdb.gimp_item_set_visible (outcome, False)
        
    outcome.name = st+oname+EFFECT+PARMS
    
    # user required output image size
    if UserOutWH==0: UserOutWH=OrigWH
    if UserOutWH != FixWH:
        uRatio = UserOutWH*1.00/FixWH*1.00
        uWidth = int(width*uRatio)
        uHeight = int(height*uRatio)
        #info_msg ("SCALING")   # not during tests, scaling is slow
        pdb.gimp_image_scale (newImage, uWidth, uHeight)   # to user chosen size
     
    pdb.gimp_display_new (newImage)

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleGraphics/"
copyrightYear="2021"
desc="Fake Pissarro (2)"

register(
    "FakePissarro2",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+" ...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
                                   "\n (0=same as input)",
                                   3000, (0, 7200, 100)),
        (PF_ADJUSTMENT, "colours", "Number of colours", 15, (3, 255, 1)),
        (PF_ADJUSTMENT, "details", "Details for Vector Paint step"+"\n (use high values for people)",
                                    9.50, (9.00, 10.00, 0.01)),
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    FakePissarro2,
    menu=menu
)

main()
